@@ -1,7 +1,7 @@
 #!/usr/bin/env perl
 use strict;
 use warnings;
-use Math::Prime::Util qw/is_aks_prime is_prime primes/;
+use Math::Prime::Util qw/is_aks_prime is_prime primes urandomm/;
 $| = 1;  # fast pipes
 
 my $limit = shift || 2_000_000_000;
@@ -22,10 +22,11 @@ print "\n";
 print "Testing $nrand random numbers from 1 to $limit:\n";
 for (1 .. $nrand) {
   print "." unless $_ % 100;
-  my $n = 1 + int(rand($limit));
+  my $n = 1 + urandomm($limit);
   if (is_prime($n)) {
     die "\n$n is prime\n" unless is_aks_prime($n);
   } else {
     die "\n$n is composite\n" if is_aks_prime($n);
   }
 }
+print "\n";
